<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use Cashbene\GatewayPrestashop\Service\InpostIntegrationService;
use Cashbene\GatewayPrestashop\Utils\CashbeneLogger;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use Cashbene\GatewayPrestashop\Service\ModuleSettingsService;
use Cashbene\GatewayPrestashop\Service\CarrierService;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

require __DIR__ . '/vendor/autoload.php';

class CashbenePaymentGateway extends PaymentModule implements WidgetInterface
{
    const MODULE_NAME = 'cashbenepaymentgateway';

    protected $config_form = false;
    /** @var string $cashbeneBoxTemplateFile */
    private $cashbeneBoxTemplateFile;
    /** @var string $cashbeneModalTemplateFile */
    private $cashbeneModalTemplateFile;
    /** @var ModuleSettingsService $moduleSettingsService */
    private $moduleSettingsService;

    public function __construct(ModuleSettingsService $moduleSettingsService, CarrierService $carrierService)
    {
        $this->module_key = 'cba2de85c6873d4c9cfcdc6c382d3c75';
        $this->name = self::MODULE_NAME;
        $this->tab = 'payments_gateways';
        $this->version = '1.0.4';
        $this->author = 'InterSynergy';
        $this->need_instance = 1;
        $this->moduleSettingsService = $moduleSettingsService;
        $this->moduleSettingsService->setModule($this);
        $this->carrierService = $carrierService;
        $this->carrierService->createCarrier();

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Cashbene Payment Gateway');
        $this->description = $this->l('The simple integration will bring a whole new dimension of payment to your store in 2 clicks. Your customers will make payment for purchases in your store within 8 seconds, reducing the likelihood of cart abandonment');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');
        $this->limited_countries = ['FR', 'PL', 'GB', 'DE'];
        $this->limited_currencies = ['EUR', 'USD', 'PLN', 'GBP'];
        $this->ps_versions_compliancy = ['min' => '1.7.7.0', 'max' => _PS_VERSION_];
        $this->cashbeneBoxTemplateFile = 'module:'.self::MODULE_NAME.'/views/templates/hook/box.tpl';
        $this->cashbeneModalTemplateFile = 'module:'.self::MODULE_NAME.'/views/templates/hook/modal.tpl';
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));
        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        InpostIntegrationService::createCashbeneTable();

        /**
         * https://www.prestashop.com/forums/topic/545927-display-custom-button-above-add-to-cart-on-product-detail-page/
         */
        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayAfterBodyOpeningTag') &&
            $this->registerHook('displayProductActions') &&
            $this->registerHook('displayAdminAfterHeader') &&
            $this->registerHook('actionAdminLoginControllerLoginAfter') &&
            (new Cashbene\GatewayPrestashop\Utils\OrderStates())->installStates() &&
            $this->registerHook('displayAdminOrderSide') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('displayExpressCheckout')
        ;
    }

    public function uninstall()
    {
        InpostIntegrationService::dropCashbeneTable();
        Configuration::deleteByName(ModuleSettingsService::CASHBENE_SETTINGS);
        Configuration::deleteByName('CASHBENE_PAYMENT_GATEWAY_ACCESS_TOKEN');
        return parent::uninstall();
    }

    public function hookDisplayAdminOrderSide($params)
    {
        $id_order = $params['id_order'];
        // Load order object
        $order = new Order((int) $id_order);

        // Validate customer object
        if (Validate::isLoadedObject($order)) {
            if ($order->module === $this->name) {
                $shipping = InpostIntegrationService::selectOrderCashbeneShipping($id_order);
                if ($shipping) {
                    global $smarty;
                    $smarty->assign('selectedLocker', $shipping);
                    return $this->display(__FILE__, 'views/templates/admin/displayAdminOrderSide.tpl');
                }
            }
        }
        return null;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitCashbene_payment_gatewayModule')) == true) {
            $this->moduleSettingsService->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        return $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCashbene_payment_gatewayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->moduleSettingsService->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        if ($this->moduleSettingsService->getMerchantShippingMethods()) {
            return $helper->generateForm([
                $this->moduleSettingsService->getConfigForm(),
                $this->moduleSettingsService->getMerchantShippingForm()
            ]);
        }
        return $helper->generateForm([$this->moduleSettingsService->getConfigForm()]);
    }

    public function hookDisplayAdminAfterHeader()
    {
        $url = $this->context->link->getAdminLink('AdminModules', true).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        if (!$this->moduleSettingsService->getMerchantShippingMethods()) {
            echo $this->displayError("<a href='$url'>" . $this->displayName . '</a> - '.$this->l('require complete the form').'.');
        } elseif ($this->moduleSettingsService->isMissingMerchantShippingConfiguration()) {
            echo $this->displayError("<a href='$url'>" . $this->displayName . '</a> - '.$this->l('Some of shipping methods are not configured').'.');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'views/js/bundle.js');
        $this->context->controller->addCSS($this->_path.'views/css/bundle.css');

        /* @var \Cashbene\Core\Utils\Configuration $cashbeneConfiguration */
        $cashbeneConfiguration = $this->get('cashbene.payment.configuration')->getConfiguration();

        $this->context->controller->registerJavascript(
            'payu-js',
            $cashbeneConfiguration->getPayuScript(),
            ['server' => 'remote']
        );

        $this->context->controller->registerJavascript(
            'inpost-js',
            $cashbeneConfiguration->getInPostScript(),
            ['server' => 'remote']
        );

        $this->context->controller->registerStylesheet(
            'inpost-css',
            $cashbeneConfiguration->getInPostStyle(),
            ['server' => 'remote']
        );
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {

    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        return null;
    }

    public function hookActionAdminLoginControllerLoginAfter()
    {
        CashbeneLogger::removeOldLogs();
    }

    public function renderWidget($hookName, array $params)
    {
        if ($this->moduleSettingsService->checkBoxWidgetRender($hookName, $params) === false) {
            return;
        }

        if (isset($params['product'])) {
            return $this->widgetForProduct($hookName, $params);
        } else {
            return $this->widgetForCart($hookName, $params);
        }
    }

    protected function widgetForProduct($hookName, $params)
    {
        $key = 'ps_'.self::MODULE_NAME.'|' . $params['product']['id_product'];
        if (!empty($params['product']['id_product_attribute'])) {
            $key .= '|' . $params['product']['id_product_attribute'];
        }

        if (!$this->isCached($this->cashbeneBoxTemplateFile, $this->getCacheId($key))) {
            $this->smarty->assign($this->getWidgetVariables($hookName, $params));
        }

        return $this->fetch($this->cashbeneBoxTemplateFile, $this->getCacheId($key));
    }

    protected function widgetForCart($hookName, $params)
    {
        $key = 'ps_'.self::MODULE_NAME.'|' . $params['cart']->id;
        if (!empty($params['cart']->getProducts())) {
            foreach ($params['cart']->getProducts() as $product) {
                if (!empty($product['id_product'])) {
                    $key .= '|' . $product['id_product'];
                }
                if (!empty($product['id_product_attribute'])) {
                    $key .= '|' . $product['id_product_attribute'];
                }
            }
        } else {
            return null;
        }

        if (!$this->isCached($this->cashbeneBoxTemplateFile, $this->getCacheId($key))) {
            $this->smarty->assign($this->getWidgetVariables($hookName, $params));
        }

        return $this->fetch($this->cashbeneBoxTemplateFile, $this->getCacheId($key));
    }

    public function hookDisplayAfterBodyOpeningTag($params)
    {
        if ($this->context->controller instanceof \ProductController) {
            // show
        } else if ($this->context->controller instanceof \CartController){
            // show
        } else {
            return;
        }

        $key = 'ps_'.self::MODULE_NAME.'|modal|' . $this->context->language->iso_code;
        if (!$this->isCached($this->cashbeneModalTemplateFile, $this->getCacheId($key))) {
            $this->smarty->assign($this->getWidgetVariables('displayFooter', $params));
        }

        return $this->fetch($this->cashbeneModalTemplateFile, $this->getCacheId($key));
    }

    public function getWidgetVariables($hookName, array $params)
    {
        $displayPage = '';
        if ($this->context->controller instanceof \ProductController) {
            $displayPage = 'product';
        } else if ($this->context->controller instanceof \CartController){
            $displayPage = 'cart';
        }

        $data = [];
        $data['lang'] = $this->context->language->iso_code === "pl" ? "pl" : "en";
        $data['modulePath'] = $this->_path;
        $data['localPath'] = $this->local_path;
        $data['cashbenePluginVersion'] = $this->version;
        $data['displayPage'] = $displayPage;
        if ($this->moduleSettingsService->getMerchantShippingMethods()) {
            $data['cashback'] = $this->get('cashbene.payment.gateway')->getGateway()->shopContext()->getMerchantCashbackValue()->value;
        }

        return $data;
    }
}
