<?php

use Cashbene\Core\Exception\HttpException;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Utils\CredentialsSession;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;
use Cashbene\GatewayPrestashop\Service\CheckoutService;
use Cashbene\GatewayPrestashop\Utils\Shop;
use Cashbene\GatewayPrestashop\Service\ProductService;

class CashbenePaymentGatewayCheckoutModuleFrontController extends CashbeneAbstractRestController
{
    const POST_ACTIONS = ['calculate', 'executeCheckout', 'calculateCart', 'executeCheckoutCart'];
    const GET_ACTIONS = ['checkoutData'];

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'checkoutData':
                return $this->getCheckoutData($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::getCheckoutData()
     */
    private function getCheckoutData(Request $request) {
        try {
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $request->get('checkoutId'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'calculate':
                return $this->calculateCart($request);
            case 'calculateCart':
                return $this->calculateCartMultipleProducts($request);
            case 'executeCheckout':
                return $this->checkoutCart($request);
            case 'executeCheckoutCart':
                return $this->checkoutCartMultipleProducts($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::calculateCartValue()
     */
    private function calculateCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $variation = $params['variationId'] ?? 0;
            $product = Shop::getProduct(Shop::getPsProduct($params['productId']), $variation);
            $product->quantity = $params['quantity'] ?? 1;
            $product = ProductService::setProductPrices($product);

            $calcOrder = CheckoutService::prepareOrder(
                [$product],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $data = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::calculateCartValue()
     */
    private function calculateCartMultipleProducts(Request $request)
    {
        $cartProducts = Context::getContext()->cart->getProducts();
        $products = [];

        foreach ($cartProducts as $productArray) {
            try {
                $products[] = ProductService::getProduct($productArray);
            } catch (\Exception $e) {
                return $this->error(new HttpException(500, "Unknown error."));
            }
        }

        try {
            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            $params = json_decode($request->getContent(), true);
            $calcOrder = CheckoutService::prepareOrder(
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $calcOrder = CheckoutService::addDiscount(
                $calcOrder,
                $products,
                $cashbeneUser,
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $data = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::checkout()
     */
    private function checkoutCart(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);

            // Prepare product
            $prestaProduct = Shop::getPsProduct($params['productId']);
            $cashbeneProduct = Shop::getProduct($prestaProduct, $params['variationId']);
            $cashbeneProduct->quantity = $params['quantity'] ?? 1;

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            $cashbeneProduct = ProductService::setProductPrices($cashbeneProduct, $cashbeneUser);

            // start prices
            $calcOrder = CheckoutService::prepareOrder(
                [$cashbeneProduct],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );
            $prices = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
            // end prices

            // Create Presta Order
            $psOrder = CheckoutService::createPrestaOrder(
                $cashbeneUser,
                [$cashbeneProduct],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                true,
                $prices
            );

            // Prepare order object
            $checkoutOrder = CheckoutService::prepareOrder(
                [$cashbeneProduct],
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                $psOrder->id . '-PS-' .
                (
                    defined('CONTAINER_TAG')
                        ? CONTAINER_TAG . '-' . self::gen_uuid()
                        : self::gen_uuid()
                )
            );

            $checkoutId = self::$cashbeneGateway->checkoutContext()->checkout(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutOrder);
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutId);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        } catch (\Exception $exception) {
            return $this->error(new HttpException($exception->getCode() ?: 500, $exception->getMessage()));
        }

        return $this->success($data, 201);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see CheckoutContext::checkout()
     */
    private function checkoutCartMultipleProducts(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $cartProducts = Context::getContext()->cart->getProducts();
            $products = [];

            foreach ($cartProducts as $productArray) {
                try {
                    $products[] = ProductService::getProduct($productArray);
                } catch (\Exception $e) {
                    return $this->error(new HttpException(500, "Unknown error."));
                }
            }

            $cashbeneUser = self::$cashbeneGateway->userContext()->getUserData(
                CredentialsSession::getCredentials(self::$cashbeneGateway)
            );

            // start prices
            $calcOrder = CheckoutService::prepareOrder(
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $calcOrder = CheckoutService::addDiscount(
                $calcOrder,
                $products,
                $cashbeneUser,
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $prices = self::$cashbeneGateway->checkoutContext()->calculateCartValue(
                CredentialsSession::getCredentials(self::$cashbeneGateway),
                $calcOrder
            );
            // end prices

            // Create Presta Order
            $psOrder = CheckoutService::createPrestaOrder(
                $cashbeneUser,
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                false,
                $prices
            );

            // Prepare order object
            $checkoutOrder = CheckoutService::prepareOrder(
                $products,
                $params['shippingAddressId'],
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId(),
                $psOrder->id . '-PS-' .
                (
                defined('CONTAINER_TAG')
                    ? CONTAINER_TAG . '-' . self::gen_uuid()
                    : self::gen_uuid()
                )
            );

            $checkoutOrder = CheckoutService::addDiscount(
                $checkoutOrder,
                $products,
                $cashbeneUser,
                $params['shippingMethodId'],
                self::$cashbeneGateway->configuration->getMerchantId()
            );

            $checkoutId = self::$cashbeneGateway->checkoutContext()->checkout(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutOrder);
            $data = self::$cashbeneGateway->checkoutContext()->getCheckoutData(CredentialsSession::getCredentials(self::$cashbeneGateway), $checkoutId);

//            CheckoutService::destroyCart();
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        } catch (\Exception $exception) {
            return $this->error(new HttpException($exception->getCode() ?: 500, $exception->getMessage()));
        }

        return $this->success($data, 201);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    private function gen_uuid() {
        return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            // 32 bits for "time_low"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

            // 16 bits for "time_mid"
            mt_rand( 0, 0xffff ),

            // 16 bits for "time_hi_and_version",
            // four most significant bits holds version number 4
            mt_rand( 0, 0x0fff ) | 0x4000,

            // 16 bits, 8 bits for "clk_seq_hi_res",
            // 8 bits for "clk_seq_low",
            // two most significant bits holds zero and one for variant DCE1.1
            mt_rand( 0, 0x3fff ) | 0x8000,

            // 48 bits for "node"
            mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
        );
    }
}
