<?php

use Cashbene\Core\Exception\HttpException;
use Cashbene\Core\Exception\InvalidOrderStatusChangeException;
use Cashbene\Core\Exception\InvalidPaidOrderStatusChangeException;
use Cashbene\Core\Exception\InvalidRequestSignatureException;
use Cashbene\Core\Exception\SignatureHeaderMissingException;
use Cashbene\Core\Exception\ValidationFailedErrorException;
use Cashbene\GatewayPrestashop\Utils\Shop as ShopUtils;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;
use Cashbene\GatewayPrestashop\Service\ProductService;
use Cashbene\GatewayPrestashop\Service\ModuleSettingsService;
use Cashbene\GatewayPrestashop\Service\ConfigurationService;
use Cashbene\GatewayPrestashop\Service\CheckoutService;
use Cashbene\Core\Utils\CredentialsSession;

class CashbenePaymentGatewayShopModuleFrontController extends CashbeneAbstractRestController
{
    const POST_ACTIONS = ['webhook', 'simulateWebhook'];
    const GET_ACTIONS = ['merchantShippingMethods', 'product', 'regions', 'cartProducts', 'verificationCode'];
    const CYPRESS_KEY = 'Cypress_Cashbene_Simulation_Secret_Key';

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'merchantShippingMethods':
                return $this->getMerchantShippingMethods();
            case 'product':
                return $this->getProduct($request);
            case 'regions':
                return $this->getRegions($request);
            case 'cartProducts':
                return $this->getCartProducts($request);
            case 'verificationCode':
                return $this->getVerificationCode($request);
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see ShopContext::getMerchantShippingMethods()
     */
    private function getMerchantShippingMethods() {
        try {
            $shippingMethods = self::$cashbeneGateway->shopContext()->getMerchantShippingMethods();
            $settings = json_decode(Configuration::get(ModuleSettingsService::CASHBENE_SETTINGS), true);

            foreach ($shippingMethods as $key => $shippingMethod) {
                if (!isset($settings['shipping_methods'][ConfigurationService::SHIPPING_METHOD_KEY.$shippingMethod->id])) {
                    unset($shippingMethods[$key]);
                }
            }
            $shippingMethods = array_values($shippingMethods);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($shippingMethods);
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see ShopContext::getVerificationCode()
     */
    private function getVerificationCode($request) {
        if (hash('sha256', self::CYPRESS_KEY) != $request->headers->get('Cypress-Key')) {
            return $this->error(
                new HttpException(404, "Not Found.")
            );
        }

        try {
            $phoneNumber = $request->query->get('phoneNumber');
            $code = self::$cashbeneGateway->shopContext()->getVerificationCode($phoneNumber);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($code);
    }

    /**
     * Return product details by product id
     *
     * @param Request $request
     * @return JsonResponse
     * @throws \Exception|\Symfony\Component\Serializer\Exception\ExceptionInterface
     * @see ShopUtils::getProduct()
     */
    private function getProduct(Request $request) {
        try {
            $productId = (int) $request->query->get('product_id');
            $variationId = (int) $request->query->get('variation_id', 0);
            $quantity = (int) $request->query->get('quantity', 1);
            $product = ShopUtils::getProduct(ShopUtils::getPsProduct($productId, $quantity), $variationId);
            $product = ProductService::setProductPrices($product);
        } catch (\Exception $e) {
            return $this->error(new HttpException(500, "Unknown error."));
        }

        return $this->success($product);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    private function getRegions(Request $request) {
        $countryCode = $request->query->get('country', 'PL');
        $regions = ShopUtils::getRegionsByCountryCode($countryCode);

        return $this->success($regions);
    }

    /**
     * @return JsonResponse
     * @throws \Symfony\Component\Serializer\Exception\ExceptionInterface
     */
    public function getCartProducts()
    {
        $cartProducts = Context::getContext()->cart->getProducts();
        $products = [];

        foreach ($cartProducts as $productArray) {
            try {
                $products[] = ProductService::getProduct($productArray);
            } catch (\Exception $e) {
                return $this->error(new HttpException(500, "Unknown error."));
            }
        }

        return $this->success($products);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'webhook':
                return $this->webhook($request);
            case 'simulateWebhook':
                return $this->simulateWebhook($request);
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see ShopContext::simulateWebhook()
     */
    private function simulateWebhook(Symfony\Component\HttpFoundation\Request $request) {
        if (hash('sha256', self::CYPRESS_KEY) != $request->headers->get('Cypress-Key')) {
            return $this->error(
                new HttpException(404, "Not Found.")
            );
        }

        $availableStatus = ['COMPLETED', 'CANCELED'];
        try {
            $jsonParams = json_decode($request->getContent(), true);
            $order = [];

            if (!isset($jsonParams['status']) || !in_array($jsonParams['status'], $availableStatus)) {
                return $this->error(
                    new HttpException(400, "Status missing or not match requirements.")
                );
            }

            if (!isset($jsonParams['orderId'])) {
                return $this->error(
                    new HttpException(400, "Missing parameter extOrderId.")
                );
            }

            $order['order']['extOrderId'] = 'CK-'.$jsonParams['orderId'];
            $order['order']['status'] = $jsonParams['status'];

            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            $code = self::$cashbeneGateway->shopContext()->simulateWebhook($credentials, $order);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($code);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    private function webhook(Request $request)
    {
        try {
            $this->logWebhookRequest($request);
            $jsonParams = json_decode($request->getContent(), true);

            if (!$request->headers->get('x_signature')) {
                $this->logWebhookError("Header: 'X-Signature' doesn't exists.");

                return $this->error(
                    new SignatureHeaderMissingException("Missing signature header.")
                );
            }

            $signatureChecker = self::$cashbeneGateway->getSignatureChecker();
            $correctlySignature = $signatureChecker->createSignature($jsonParams);
            $isValidSignatureRequest = $signatureChecker->compareSignatures(
                $request->headers->get('x_signature'),
                $correctlySignature
            );

            if (!$isValidSignatureRequest) {
                $this->logWebhookError("The request signature is invalid.");

                return $this->error(
                    new InvalidRequestSignatureException("The request signature is invalid.")
                );
            }

            // Checks whether the required parameters exist
            $validationsError = [];

            if (!isset($jsonParams['status'])) {
                $this->logWebhookError("Param: 'status' doesn't exists.");

                $validationsError[] = [
                    "field" => "status",
                    "message" => "must not be null"
                ];
            }

            if (!isset($jsonParams['extOrderId'])) {
                $this->logWebhookError("Param: 'extOrderId' doesn't exists.");

                $validationsError[] = [
                    "field" => "extOrderId",
                    "message" => "must not be null"
                ];
            }

            if (!empty($validationsError)) {
                return $this->error(
                    (new ValidationFailedErrorException("Request parameters constraints violation"))
                        ->setErrors($validationsError)
                );
            }

            // Checks whether the status and order are valid
            $statusId = ShopUtils::getPsStatusIdByCashbeneStatus($jsonParams['status']);
            if ($statusId === false) {
                $this->logWebhookError("Unknown status: '{$jsonParams['status']}'");

                $validationsError[] = [
                    "field" => "status",
                    "message" => "unknown status: '{$jsonParams['status']}', must be 'SUCCESSFUL' or 'FAIL'"
                ];
            }

            $orderId = (int) substr(
                $jsonParams['extOrderId'],
                0,
                strpos($jsonParams['extOrderId'], '-')
            );

            $order = new Order($orderId);
            if ($order === false || $order->module !== CashbenePaymentGateway::MODULE_NAME) {
                $this->logWebhookError("Unknown order by ID: '{$jsonParams['extOrderId']}'");

                $validationsError[] = [
                    "field" => "extOrderId",
                    "message" => "order with the number '$orderId' does not exist"
                ];
            }

            if (!empty($validationsError)) {
                return $this->error(
                    (new ValidationFailedErrorException("Request parameters constraints violation"))
                        ->setErrors($validationsError)
                );
            }

            $orderStateId = $order->getCurrentOrderState()->id;
            $orderStates = Configuration::getMultiple([
                'PS_CHECKOUT_STATE_WAITING_CASHBENE_PAYMENT',
                'PS_OS_ERROR',
                'PS_OS_PAYMENT',
                'PS_OS_PREPARATION',
                'PS_OS_SHIPPING',
                'PS_OS_DELIVERED',
                'PS_OS_OUTOFSTOCK_PAID'
            ]);

            if (
                $orderStateId == $orderStates['PS_CHECKOUT_STATE_WAITING_CASHBENE_PAYMENT']
                || $orderStateId == $orderStates['PS_OS_ERROR']
            ) {
                $order->setCurrentState($statusId);
                $order->save();
            } elseif (
                $orderStateId == $orderStates['PS_OS_PAYMENT']
                || $orderStateId == $orderStates['PS_OS_PREPARATION']
                || $orderStateId == $orderStates['PS_OS_SHIPPING']
                || $orderStateId == $orderStates['PS_OS_DELIVERED']
                || $orderStateId == $orderStates['PS_OS_OUTOFSTOCK_PAID']
            ) {
                return $this->error(new InvalidPaidOrderStatusChangeException("Invalid status change for a paid order."));
            } else {
                return $this->error(new InvalidOrderStatusChangeException("Invalid status change for an order."));
            }

            return $this->success();

        } catch (\Exception $e) {
            $this->logWebhookError(
                print_r([
                    'message' => $e->getMessage(),
                    'file' => $e->getFile(),
                    'line' => $e->getLine(),
                    'code' => $e->getCode(),
                    'trace' => $e->getTrace()
                ], true)
            );

            return $this->error(new HttpException(500, "Unknown error."));
        }
    }

    /**
     * @param string $message
     * @return void
     */
    private function logWebhookError(string $message)
    {
        $this->logger->setFilename('webhook');
        $this->logger->logError($message);
    }

    /**
     * @param Request $request
     * @return void
     */
    private function logWebhookRequest(Request $request)
    {
        $this->logger->setFilename('webhook');
        $this->logger->logInfo(
            print_r([
                'json_params' => json_decode($request->getContent(), true),
                'headers' => $request->headers
            ], true)
        );
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }
}
