<?php

use Cashbene\Core\Dto\ShippingMethod;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Utils\CredentialsSession;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;

class CashbenePaymentGatewayUserModuleFrontController extends CashbeneAbstractRestController
{
    const POST_ACTIONS = ['signIn', 'shipmentAddresses'];
    const GET_ACTIONS = ['users', 'shipmentAddresses'];
    const DELETE_ACTIONS = ['shipmentAddresses'];
    const PUT_ACTIONS = ['shipmentAddresses'];

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'users':
                return $this->getUserData();
            case 'shipmentAddresses':
                return $this->getShipmentAddresses($request);
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::getUserData()
     */
    private function getUserData() {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            $data = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::getShipmentAddresses()
     */
    private function getShipmentAddresses(Request $request) {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            $data = self::$cashbeneGateway
                ->userContext()
                ->getShipmentAddresses($credentials, $userData->uuid, $request->get('type') ?? null);

        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'signIn':
                return $this->signIn($request);
            case 'shipmentAddresses':
                return $this->addShipmentAddress($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::signIn()
     */
    private function signIn(Request $request) {
        $data = json_decode($request->getContent(), true);

        try {
            $clientCredentials = self::$cashbeneGateway->userContext()->signIn($data['email'], $data['password']);
            CredentialsSession::saveCredentials($clientCredentials);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success();
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::addShipmentAddress()
     */
    private function addShipmentAddress(Request $request) {
        $data = json_decode($request->getContent(), true);

        /** @var ShippingMethod $shippingMethod */
        $shippingMethod = self::$serializer->denormalize(
            $data, ShippingMethod::class, 'array'
        );

        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            self::$cashbeneGateway
                ->userContext()
                ->addShipmentAddress($credentials, $shippingMethod);

        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 201);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'shipmentAddresses':
                return $this->updateShipmentAddress($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::updateShipmentAddress()
     */
    public function updateShipmentAddress(Request $request)
    {
        $data = json_decode($request->getContent(), true);

        /** @var ShippingMethod $shippingMethod */
        $shippingMethod = self::$serializer->denormalize(
            $data, ShippingMethod::class, 'array'
        );

        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            self::$cashbeneGateway
                ->userContext()
                ->updateShipmentAddress($credentials, $userData->uuid, $request->get('addressUuid'), $shippingMethod);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success();
    }


    protected function processDeleteRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'shipmentAddresses':
                return $this->deleteShipmentAddress($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see UserContext::deleteShipmentAddress()
     */
    public function deleteShipmentAddress(Request $request)
    {
        try {
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);

            /** @var \Cashbene\Core\Dto\User $userData */
            $userData = self::$cashbeneGateway
                ->userContext()
                ->getUserData($credentials);

            self::$cashbeneGateway
                ->userContext()
                ->deleteShipmentAddress($credentials, $userData->uuid, $request->get('addressUuid'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 204);
    }

}
