<?php
namespace Cashbene\GatewayPrestashop\Rest;

use FileLogger;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Exception\NotFoundHttpException;
use Cashbene\Core\Exception\ParamMismatchException;
use Cashbene\Core\Exception\ParamMissingException;
use Cashbene\Core\Service\HttpExceptionService;
use Cashbene\Core\Service\Response;
use Cashbene\Core\Service\Validator;
use Cashbene\GatewayPrestashop\Utils\CashbeneLogger;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use ModuleFrontController;
use Cashbene\Core\Service\SerializerService;

abstract class CashbeneAbstractRestController extends ModuleFrontController
{
    public static $cashbeneGateway;

    /** @var \Symfony\Component\Serializer\Serializer */
    public static $serializer;

    /** @var CashbeneLogger */
    protected $logger;

    public function __construct()
    {
        $this->logger = new CashbeneLogger;
        $this->logger->setFilename('rest');

        parent::__construct();
    }

    public function init()
    {
        parent::init();
        session_start();
        $this->ajax = true;
        self::$cashbeneGateway = $this->get('cashbene.payment.gateway')->getGateway();
        self::$serializer = SerializerService::getSerializer();

        $request = Request::createFromGlobals();
        $action = $request->get('action');

        $httpMethod = strtoupper($request->getMethod());
        $actions = $httpMethod . '_ACTIONS';
        $reflection = new \ReflectionClass(get_class($this));
        $actions = $reflection->getConstant($actions);

        if (!$action || !in_array($action, $actions)) {
            $locale = $this->get('cashbene.payment.configuration')
                ->getConfiguration()
                ->getLanguage();

            $validator = new Validator(
                new HttpExceptionService(),
                $locale
            );

            if (!$action) {
                return $this->error(
                    $validator->translateException(
                        new ParamMissingException("Param: 'action' missing.")
                    )
                );
            } else if (!in_array($action, $actions)) {
                return $this->error(
                    $validator->translateException(
                        new ParamMismatchException("Param: 'action' mismatch.")
                    )
                );
            }
        }

        switch ($httpMethod) {
            case 'GET':
                $response = $this->processGetRequest($action, $request);
                break;
            case 'POST':
                $response = $this->processPostRequest($action, $request);
                break;
            case 'PATCH':
            case 'PUT':
                $response = $this->processPutRequest($action, $request);
                break;
            case 'DELETE':
                $response = $this->processDeleteRequest($action, $request);
                break;
        }

        if (empty($response)) {
            $this->error(new NotFoundHttpException("Not found."));
        }
    }

    abstract protected function processGetRequest(string $action, Request $request);
    abstract protected function processPostRequest(string $action, Request $request);
    abstract protected function processPutRequest(string $action, Request $request);
    abstract protected function processDeleteRequest(string $action, Request $request);

    /**
     * Returns JsonResponse object based on the contents of a given exception.
     *
     * @param HttpExceptionInterface $httpException
     * @param $code
     * @return JsonResponse
     */
    public function error(HttpExceptionInterface $httpException, $code = null)
    {
        $responseBody = Response::prepareSimpleErrorResponseBody($httpException);

        if (is_null($code)) {
            $code = $httpException->getHttpCode();
        }

        return (new JsonResponse($responseBody, $code))->send();
    }

    /**
     * Returns JsonResponse object with given data and http status code.
     *
     * @param array|object|null $data
     * @param $code
     * @return JsonResponse
     */
    public function success($data = null, $code = 200)
    {
        $responseBody = Response::prepareSimpleSuccessResponseBody($data);
        return (new JsonResponse($responseBody, $code))->send();
    }
}
