<?php

namespace Cashbene\GatewayPrestashop\Service;

use Carrier;
use CMS;
use Configuration;
use Context;
use Cashbene\Core\Exception\HttpException;
use Tools;

class ModuleSettingsService
{
    const CASHBENE_SETTINGS = 'CASHBENE_PAYMENT_GATEWAY_SETTINGS';

    private $module;

    public function setModule($module) {
        $this->module = $module;
    }

    public function __construct(GatewayService $gateway, ConfigurationService $configurationService)
    {
        $this->module = null;
        $this->gatewayService = $gateway;
        $this->gateway = $gateway->getGateway();
        $this->configurationService = $configurationService;
        $this->context = Context::getContext();
    }

    /**
     * Define default setting fields
     * @return array
     */
    public static function settingFields() {
        return [
            'CASHBENE_PAYMENT_GATEWAY_MERCHANT_ID' => '',
            'CASHBENE_PAYMENT_GATEWAY_CLIENT_ID' => '',
            'CASHBENE_PAYMENT_GATEWAY_CLIENT_SECRET' => '',
            'CASHBENE_PAYMENT_GATEWAY_SECRET_KEY' => '',
            'CASHBENE_PAYMENT_GATEWAY_ENVIRONMENT' => '',
            'CASHBENE_PAYMENT_GATEWAY_TERMS_AND_CONDITIONS_PAGE' => '',
            'CASHBENE_PAYMENT_GATEWAY_PRIVACY_POLICY_PAGE' => ''
        ];
    }

    public function getMerchantShippingMethods()
    {
        try {
            $cashbeneShippingMethods = $this->gateway->shopContext()->getMerchantShippingMethods();
        } catch (HttpException $httpException) {
            return null;
        }

        return $cashbeneShippingMethods;
    }

    public function getMerchantShippingForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->module->l('Merchant Shipping', 'modulesettingsservice'),
                    'icon' => 'icon-truck',
                ],
                'input' => [],
                'submit' => [
                    'title' => $this->module->l('Save', 'modulesettingsservice'),
                ],
            ],
        ];

        $cashbeneShippingMethods = $this->getMerchantShippingMethods();
        $shippingMethods = $this->getShippingMethods();

        if ($cashbeneShippingMethods) {
            foreach ($cashbeneShippingMethods as $cashbeneShippingMethod) {
                if(!$cashbeneShippingMethod->active) continue;
                
                $shippingMethodsArray = [];
                foreach ($shippingMethods as $key => $value) {
                    $shippingMethodsArray[$key]['id'] = $key;
                    $shippingMethodsArray[$key]['name'] = $value;
                }

                $fields_form['form']['input'][] = [
                    'col' => 3,
                    'type' => 'select',
                    'name' => ConfigurationService::SHIPPING_METHOD_KEY.$cashbeneShippingMethod->id,
                    'label' => $this->module->l($cashbeneShippingMethod->name),
                    'options' => [
                        'query' => $shippingMethodsArray,
                        'id' => 'id',
                        'name' => 'name',
                        'default' => [
                            'value' => '',
                            'label' => $this->module->l('Select', 'modulesettingsservice')
                        ]
                    ],
                ];
            }
        }

        return $fields_form;
    }

    /**
     * Create the structure of your form.
     */
    public function getConfigForm()
    {
        $pages = CMS::getCMSPages($this->context->language->id);

        return [
             'form' => [
                'legend' => [
                    'title' => $this->module->l('Settings', 'modulesettingsservice'),
                    'icon' => 'icon-cogs',
                ],
                 'description' => 'Jeśli występuję problemy z korzystaniem z wtyczki, proszę wysłać zgłoszenie za pośrednictwem <a href="https://pacificorg.atlassian.net/servicedesk/customer/portal/4" target="_blank">formularza wsparcia technicznego.</a>',
                'input' => [
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_MERCHANT_ID',
                        'label' => $this->module->l('Merchant ID', 'modulesettingsservice'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_CLIENT_ID',
                        'label' => $this->module->l('Client ID', 'modulesettingsservice'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'password',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_CLIENT_SECRET',
                        'label' => $this->module->l('Client secret'),
                        'desc' => empty($this->getModuleSettings()['CASHBENE_PAYMENT_GATEWAY_CLIENT_SECRET']) ? $this->module->l('empty', 'modulesettingsservice') : $this->module->l('filled', 'modulesettingsservice'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'password',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_SECRET_KEY',
                        'label' => $this->module->l('Secret key'),
                        'desc' => empty($this->getModuleSettings()['CASHBENE_PAYMENT_GATEWAY_SECRET_KEY']) ? $this->module->l('empty', 'modulesettingsservice') : $this->module->l('filled', 'modulesettingsservice'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'select',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_ENVIRONMENT',
                        'label' => $this->module->l('Environment', 'modulesettingsservice'),
                        'options' => [
                            'query' => [
                                ['id' => 'sandbox', 'name' => $this->module->l('Sandbox', 'modulesettingsservice')],
                                ['id' => 'production', 'name' => $this->module->l('Production', 'modulesettingsservice')],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'col' => 3,
                        'type' => 'select',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_TERMS_AND_CONDITIONS_PAGE',
                        'label' => $this->module->l('Terms and conditions page', 'modulesettingsservice'),
                        'options' => [
                            'query' => $pages,
                            'id' => 'id_cms',
                            'name' => 'meta_title',
                            'default' => [
                                'value' => '',
                                'label' => $this->module->l('Select', 'modulesettingsservice')
                            ]
                        ]
                    ],
                    [
                        'col' => 3,
                        'type' => 'select',
                        'name' => 'CASHBENE_PAYMENT_GATEWAY_PRIVACY_POLICY_PAGE',
                        'label' => $this->module->l('Privacy policy page', 'modulesettingsservice'),
                        'options' => [
                            'query' => $pages,
                            'id' => 'id_cms',
                            'name' => 'meta_title',
                            'default' => [
                                'value' => '',
                                'label' => $this->module->l('Select', 'modulesettingsservice')
                            ]
                        ]
                    ],
                ],
                'submit' => [
                    'title' => $this->module->l('Save', 'modulesettingsservice'),
                ],
            ]
        ];
    }

    /**
     * @return array
     */
    private function getShippingMethods()
    {
        $registeredShippingMethods = Carrier::getCarriers($this->context->language->id);

        $shippingMethods = [];
        foreach ($registeredShippingMethods as $shippingMethod) {
            $shippingMethods[$shippingMethod['id_carrier']] = $shippingMethod['name'];
        }

        return $shippingMethods;
    }

    public static function getModuleSettings() {
        $settings = json_decode(Configuration::get(self::CASHBENE_SETTINGS), true);
        $defaultSettings = self::settingFields();
        return array_merge($defaultSettings, $settings ?? []);
    }

    /**
     * Set values for the inputs.
     */
    public function getConfigFormValues()
    {
        $settings = $this->getModuleSettings();
        $formValues = [];
        $formValues = $this->getShippingMethodFormValues($formValues, $settings);

        foreach (self::settingFields() as $FieldKey => $defaultValue) {
            $formValues[$FieldKey] = $settings[$FieldKey];
        }
        return $formValues;
    }

    /**
     * Set values for the inputs.
     */
    protected function getShippingMethodFormValues($formValues, $settings)
    {
        $cashbeneShippingMethods = $this->getMerchantShippingMethods();

        if ($cashbeneShippingMethods) {
            foreach ($cashbeneShippingMethods as $cashbeneShippingMethod) {
                $field = ConfigurationService::SHIPPING_METHOD_KEY.$cashbeneShippingMethod->id;
                $formValues[$field] = $settings['shipping_methods'][$field] ?? null;
            }
        }

        return $formValues;
    }

    /**
     * Save form data.
     */
    public function postProcess()
    {
        $form_values = $this->getConfigFormValues();
        $settings = [];

        foreach (array_keys($form_values) as $key) {
            // Password fields never displays anything, so need to update only when it's filled.
            // https://www.prestashop.com/forums/topic/347850-possible-bug-with-helperform-and-password-type-fields/
            if (in_array($key, ['CASHBENE_PAYMENT_GATEWAY_CLIENT_SECRET', 'CASHBENE_PAYMENT_GATEWAY_SECRET_KEY'])) {
                if (strval(Tools::getValue($key)) != "") {
                    $settings[$key] = Tools::getValue($key);
                } else {
                    $settings[$key] = $this->getModuleSettings()[$key];
                }
            } else {
                if (str_contains($key, 'SHIPPING_METHODS')) {
                    if (strval(Tools::getValue($key)) != "") {
                        $settings['shipping_methods'][$key] = Tools::getValue($key);
                    }
                } else {
                    $settings[$key] = Tools::getValue($key);
                }
            }
        }

        Configuration::updateValue(self::CASHBENE_SETTINGS, json_encode($settings));
        $this->reloadGateway();
    }

    private function reloadGateway() {
        $this->configurationService->reloadConfigurationSettings();
        $this->gatewayService->reloadGatewayInstance($this->configurationService);
        $this->gateway = $this->gatewayService->getGateway();
    }

    public function checkBoxWidgetRender($hookName, $params) {

        // check shipping is set
        if (!isset($this->getModuleSettings()['shipping_methods'])) {
            return false;
        }

        if (method_exists($this->context->controller, 'getProduct')) {
            if ($hookName != 'displayProductActions') {
                return false;
            }

            // check if is quick view
            if ($this->context->controller->isQuickView() === true) {
                return false;
            }
        } else {
            if ($hookName != 'displayExpressCheckout') {
                return false;
            }
        }

        return true;
    }

    public function isMissingMerchantShippingConfiguration()
    {
        $cashbeneShippingMethods = $this->getMerchantShippingMethods();
        $config = $this->getConfigFormValues();

        $missingShippingConfig = [];
        if ($cashbeneShippingMethods) {
            foreach ($cashbeneShippingMethods as $cashbeneShippingMethod) {
                if (!$cashbeneShippingMethod->active) continue;

                $shippingConfigKey = ConfigurationService::SHIPPING_METHOD_KEY.$cashbeneShippingMethod->id;
                if (isset($config[$shippingConfigKey]) && $config[$shippingConfigKey] != null) {

                } else {
                    $missingShippingConfig[] = $cashbeneShippingMethod->name;
                }
            }
        }

        if (!empty($missingShippingConfig)) {
            return true;
        }
        return false;
    }

}
