<?php

namespace Cashbene\GatewayPrestashop\Utils;

use PrestaShop\Module\PrestashopCheckout\Exception\PsCheckoutException;
use Cashbene\GatewayPrestashop\Translations\OrderStatesTranslations;
use CashbenePaymentGateway;

class OrderStates
{
    const ORDER_STATE_TEMPLATE = 'payment';
    const ORDER_STATE_TABLE = 'order_state';
    const ORDER_STATE_LANG_TABLE = 'order_state_lang';
    const DARK_BLUE_HEXA_COLOR = '#34209E';
    const ORDER_STATES = [
        'PS_CHECKOUT_STATE_WAITING_CASHBENE_PAYMENT' => self::DARK_BLUE_HEXA_COLOR,
    ];

    /**
     * Insert the new states if it does not exists
     * Create a new order state for each ps_checkout new order states
     *
     * FYI: this method is also used in the upgrade-1.2.14.php file
     *
     * @return bool
     *
     * @throws PsCheckoutException
     * @throws \PrestaShopDatabaseException
     */
    public function installStates()
    {
        foreach (self::ORDER_STATES as $state => $color) {
            $orderStateId = $this->getStateId($state, $color);
            $this->createStateLangs($state, $orderStateId);
            $this->setStateIcons($state, $orderStateId);
        }

        return true;
    }

    /**
     * Get the state id if it already exist.
     * Get the state id if it doesn't exist by creating it
     *
     * @param string $state
     * @param string $color
     *
     * @return int
     *
     * @throws PsCheckoutException
     * @throws \PrestaShopDatabaseException
     */
    private function getStateId($state, $color)
    {
        $stateId = (int) \Configuration::getGlobalValue($state);

        // Is state ID already existing in the Configuration table ?
        if (0 === $stateId || false === $this->stateAlreadyExists($stateId)) {
            return $this->createStateId($state, $color);
        }

        return (int) $stateId;
    }

    /**
     * Create the State id
     *
     * @param string $state
     * @param string $color
     *
     * @return int orderStateId
     *
     * @throws PsCheckoutException
     * @throws \PrestaShopDatabaseException
     */
    private function createStateId($state, $color)
    {
        $data = [
            'module_name' => CashbenePaymentGateway::MODULE_NAME,
            'color' => $color,
            'unremovable' => 1,
        ];

        if (true === \Db::getInstance()->insert(self::ORDER_STATE_TABLE, $data)) {
            $insertedId = (int) \Db::getInstance()->Insert_ID();
            \Configuration::updateGlobalValue($state, $insertedId);

            return $insertedId;
        }

        throw new PsCheckoutException('Not able to insert the new order state', PsCheckoutException::PRESTASHOP_ORDER_STATE_ERROR);
    }

    /**
     * Create the States Lang
     *
     * @param string $state
     * @param int $orderStateId
     *
     * @throws PsCheckoutException
     * @throws \PrestaShopDatabaseException
     */
    private function createStateLangs($state, $orderStateId)
    {
        $languagesList = \Language::getLanguages();
        $orderStatesTranslations = new OrderStatesTranslations();

        // For each languages in the shop, we insert a new order state name
        foreach ($languagesList as $key => $lang) {
            if (true === $this->stateLangAlreadyExists($orderStateId, (int) $lang['id_lang'])) {
                continue;
            }

            $statesTranslations = $orderStatesTranslations->getTranslations($lang['iso_code']);
            $this->insertNewStateLang($orderStateId, $statesTranslations[$state], (int) $lang['id_lang']);
        }
    }

    /**
     * Check if State language already exists in the table ORDEr_STATE_LANG_TABLE
     *
     * @param int $orderStateId
     * @param int $langId
     *
     * @return bool
     */
    private function stateLangAlreadyExists($orderStateId, $langId)
    {
        return (bool) \Db::getInstance()->getValue(
            'SELECT id_order_state
            FROM  `' . _DB_PREFIX_ . self::ORDER_STATE_LANG_TABLE . '`
            WHERE
                id_order_state = ' . $orderStateId . '
                AND id_lang = ' . $langId
        );
    }

    /**
     * Create the States Lang
     *
     * @param int $orderStateId
     * @param string $translations
     * @param int $langId
     *
     * @throws PsCheckoutException
     * @throws \PrestaShopDatabaseException
     */
    private function insertNewStateLang($orderStateId, $translations, $langId)
    {
        $data = [
            'id_order_state' => $orderStateId,
            'id_lang' => (int) $langId,
            'name' => pSQL($translations),
            'template' => self::ORDER_STATE_TEMPLATE,
        ];

        if (false === \Db::getInstance()->insert(self::ORDER_STATE_LANG_TABLE, $data)) {
            throw new PsCheckoutException('Not able to insert the new order state language', PsCheckoutException::PRESTASHOP_ORDER_STATE_ERROR);
        }
    }

    /**
     * Set an icon for the current State Id
     *
     * @param string $state
     * @param int $orderStateId
     *
     * @return bool
     */
    private function setStateIcons($state, $orderStateId)
    {
        /** @var \CashbenePaymentGateway $module */
        $module = \Module::getInstanceByName(CashbenePaymentGateway::MODULE_NAME);
        $iconExtension = '.gif';
        $iconToPaste = _PS_ORDER_STATE_IMG_DIR_ . $orderStateId . $iconExtension;

        if (true === file_exists($iconToPaste)) {
            if (true !== is_writable($iconToPaste)) {
//                $module->getLogger()->error(
//                    '[PSPInstall] file is not writable',
//                    [
//                        'file' => $iconToPaste,
//                    ]
//                );

                return false;
            }
        }

        $iconName = 'waiting';

        $iconsFolderOrigin = _PS_MODULE_DIR_ . CashbenePaymentGateway::MODULE_NAME . '/views/img/OrderStatesIcons/';
        $iconToCopy = $iconsFolderOrigin . $iconName . $iconExtension;

        if (false === copy($iconToCopy, $iconToPaste)) {
//            $module->getLogger()->error(
//                '[PSPInstall] not able to copy icon',
//                [
//                    'icon' => $iconName,
//                    'id_order_state' => $orderStateId,
//                ]
//            );

            return false;
        }

        return true;
    }

    /**
     * Check if OrderState already exists in the table
     *
     * @param int $orderStateId
     *
     * @return bool
     */
    private function stateAlreadyExists($orderStateId)
    {
        $query = new \DbQuery();
        $query->select('id_order_state');
        $query->from(self::ORDER_STATE_TABLE);
        $query->where('id_order_state = ' . (int) $orderStateId);
        $query->where('module_name = "' . CashbenePaymentGateway::MODULE_NAME . '"');

        return (bool) \Db::getInstance()->getValue($query);
    }
}
