<?php

namespace Cashbene\GatewayPrestashop\Utils;

use Configuration;
use Context;
use Image;
use Cashbene\Core\Dto\Shop\Attribute;
use Cashbene\Core\Dto\Shop\Price;
use Cashbene\Core\Dto\Shop\Product as DtoProduct;
use Cashbene\Core\Service\SerializerService;
use Product;
use Tools;

final class Shop {

    /**
     * @param Product $psProduct
     * @param int $variationId
     * @param $country
     * @return DtoProduct
     * @throws \Symfony\Component\Serializer\Exception\ExceptionInterface
     */
	static public function getProduct(Product $psProduct, int $variationId = 0, $country = 'pl'): DtoProduct {
        $taxEnabled = Product::$_taxCalculationMethod == PS_TAX_INC;

		$attributes = self::getAttributesArray($psProduct, $variationId);
		$cashbenePrice = new Price();
		$cashbenePrice->amount = self::getDisplayPrice($psProduct->id, $variationId, $taxEnabled);
		$cashbenePrice->currency = Context::getContext()->currency->iso_code;
		$cashbenePrice->setHasTax($taxEnabled);
		$cashbenePrice->setTaxEnabled((bool) Configuration::get('PS_TAX'));

		if($taxEnabled) {
            $cashbenePrice->setTaxRate($psProduct->getTaxesRate());
		}

        $mainImage = self::getProductImage($psProduct->id, $variationId);

		$cashbeneProduct = new DtoProduct();
		$cashbeneProduct->sku = (string) ($psProduct->reference ?: $variationId ?: $psProduct->id);
		$cashbeneProduct->unitPrice = $cashbenePrice;
        $cashbeneProduct->unitGrossPrice = clone $cashbenePrice;
        $cashbeneProduct->unitGrossPrice->amount = self::getDisplayPriceNotRound($psProduct->id, $variationId, true);
        $cashbeneProduct->totalPrice = $cashbenePrice;
        $cashbeneProduct->displayPrice = $cashbenePrice;

        $cashbeneProduct->quantity = $psProduct->quantity;
		$cashbeneProduct->name = $psProduct->name;
		$cashbeneProduct->externalProductId = $variationId;
		$cashbeneProduct->image = !empty($mainImage)
            ? Context::getContext()->link->getImageLink($psProduct->link_rewrite ?: $psProduct->name, $mainImage['id_image'])
            : null;

		if($attributes) {
			$cashbeneProduct->setAllAttributes($attributes);
			$cashbeneProduct->attributes = SerializerService::getSerializer()->denormalize($attributes, Attribute::class.'[]','array');
		}

        $cashbeneProduct->productId = $psProduct->id;

		return $cashbeneProduct;
	}

    private static function getProductImage($productId, $variationId) {
        $images = Image::getImages(Context::getContext()->language->id, $productId, $variationId);

        // if variation have no images get product images
        if (empty($images)) {
            $images = Image::getImages(Context::getContext()->language->id, $productId);
        }

        $mainImage = current(
            array_filter($images, function ($image) {
                return $image['cover'];
            })
        );

        if (empty($mainImage)) {
            $pos = null;
            foreach ($images as $image) {
                if ($pos == null || $pos > $image['position']) {
                    $pos = $image['position'];
                    $mainImage = $image;
                }
            }
        }

        return $mainImage;
    }

    /**
     * @param $id
     * @return Product|null
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
	public static function getPsProduct(int $id, $quantity = 1)
    {
        $product = null;

        if (Product::existsInDatabase($id, 'product')) {
            $product = new Product($id, false, Context::getContext()->language->id);
            $product->quantity = (int) $quantity;
        };

		return $product;
	}

    /**
     * @param Product $product
     * @param int $productAttributeId
     * @return array
     */
	public static function getAttributesArray(Product $product, int $productAttributeId) : array
	{
        $combinations = $product->getAttributeCombinations();
        $attributes = [];

        foreach($combinations as $combination) {
            if ($combination['id_product_attribute'] == $productAttributeId) {
                $attributes[] = [
                    'key' => $combination['id_attribute'],
                    'name' => $combination['group_name'],
                    'value' => $combination['attribute_name'],
                    'rawValue' => $combination['attribute_name'],
                    'partOfName' => false
                ];
            }
        }

		return $attributes;
	}

    public static function getDisplayPrice($productId, $variationId, $useTax = true)
    {
        return Tools::ps_round(
            Product::getPriceStatic($productId, $useTax, $variationId),
            Context::getContext()->currency->precision
        );
    }

    public static function getDisplayPriceNotRound($productId, $variationId, $useTax = true)
    {
        return Product::getPriceStatic($productId, $useTax, $variationId);
    }

	/**
	 * @param string $status
	 * @return false|string
	 */
    public static function getPsStatusIdByCashbeneStatus(string $status)
    {
        switch ($status) {
//            case "pending":
//                return "pending";
            case "SUCCESSFUL":
                return \Configuration::get("PS_OS_PAYMENT");
//            case "on-hold":
//                return "on-hold";
//            case "completed":
//                return "completed";
//            case "cancelled":
//                return "cancelled";
//            case "refunded":
//                return "refunded";
            case "FAILED":
                return \Configuration::get("PS_OS_ERROR");
            default:
                return false;
        }
    }

    /**
     * @param string $countryCode
     * @return array|string[]
     */
    public static function getRegionsByCountryCode(string $countryCode): array
    {
        $data = [];
        if ($countryCode == 'PL') {
            $countryList = [
                'dolnośląskie',
                'kujawsko-pomorskie',
                'lubelskie',
                'lubuskie',
                'łódzkie',
                'małopolskie',
                'mazowieckie',
                'opolskie',
                'podkarpackie',
                'podlaskie',
                'pomorskie',
                'śląskie',
                'świętokrzyskie',
                'warmińsko-mazurskie',
                'wielkopolskie',
                'zachodniopomorskie'
            ];
            $i = 1;
            foreach ($countryList as $country) {
                $data[$i]['id'] = $i;
                $data[$i]['value'] = $country;
                $data[$i]['country'] = 'PL';
                $i++;
            }
        } else {
            $data[] = 'inne';
        }

        return $data;
    }

    public static function setProductTotalPrice(DtoProduct $product, $productArray)
    {
        $totalPrice = new Price();
        $totalPrice->amount = $productArray['total_wt'];
        $totalPrice->setTaxRate($productArray['rate']);
        $totalPrice->currency = Context::getContext()->currency->iso_code;
        $totalPrice->setTaxEnabled((bool) Configuration::get('PS_TAX'));
        $totalPrice->setHasTax((bool) Configuration::get('PS_TAX'));
        $product->totalPrice = $totalPrice;

        return $product;
    }

    public static function setProductDisplayPrice(DtoProduct $product, $productArray)
    {
        $taxEnabled = Product::$_taxCalculationMethod == PS_TAX_INC;
        $displayPrice = new Price();

        if ($taxEnabled) {
            $displayPrice->amount = $productArray['total_wt'];
        } else {
            $displayPrice->amount = $productArray['total'];
        }

        $displayPrice->setTaxRate($productArray['rate']);
        $displayPrice->currency = Context::getContext()->currency->iso_code;
        $displayPrice->setTaxEnabled((bool) Configuration::get('PS_TAX'));
        $displayPrice->setHasTax($taxEnabled);
        $product->displayPrice = $displayPrice;

        return $product;
    }
}
